from typing import Union
from discord_slash.model import SlashCommandPermissionType
from discord_slash.utils.manage_commands import create_permission

class Permissions:
    """Creates a slash command permissions template

    ### Args:
        `guild_id (int, list[int])`: List of guild ids
    
    ### Example: ::

        permissions = Permissions([123, 456, ...])
    """

    def __init__(self, guild_id:Union[int, list[int]]) -> None:
        self.guild_ids = [guild_id] if isinstance(guild_id, int) else guild_id
        self.permissions = {}
        for id in self.guild_ids:
            self.permissions[id] = []
    
    def everyone_permission(self, allow:bool=False) -> dict:
        """Allow or deny permission to @everyone

        ### Args:
            `allow (bool, optional)`: Whether to allow or deny permission. Defaults to False.
        
        ### Returns:
            `dict`: Permissions
        
        ### Example: ::

            permissions = Permissions(...)
            permissions.everyone_permission(False)
        """
        for id in self.guild_ids:
            permission = create_permission(id, SlashCommandPermissionType.ROLE, allow)
            if permission not in self.permissions[id]:
                self.permissions[id].append(permission)
        return self.permissions
    
    def allow_users(self, users:list[int], allow:bool=True) -> dict:
        """Allow or deny permissions to a list of user ids

        ### Args:
            `users (list[int])`: List of user ids
            `allow (bool, optional)`: Whether to allow or deny permission. Defaults to True.
        
        ### Returns:
            `dict`: Permissions
        
        ### Example: ::

            permissions = Permissions(...)
            permissions.allow_users([123, 456, ...])
        """
        for id in self.guild_ids:
            for user in users:
                self.permissions[id].append(create_permission(user, SlashCommandPermissionType.USER, allow))
        return self.permissions
    
    def allow_only_users(self, users:list[int]) -> dict:
        """Deny permissions for @everyone and allow them to a list of users ids

        ### Args:
            `users (list[int])`: List of users ids
        
        ### Returns:
            `dict`: Permissions

        ### Example: ::

            permissions = Permissions(...)
            permissions.allow_only_users([123, 456, ...])
        
        ### Equivalent to: ::

            permissions = Permissions(...)
            permissions.everyone_permission(False)
            permissions.allow_users([123, 456, ...])
        """
        self.everyone_permission(False)
        self.allow_users(users)
        return self.permissions
    
    def deny_users(self, users:list[int]) -> dict:
        """Deny permissions for a list of user ids, same as `allow_users(..., False)`

        ### Args:
            `users (list[int])`: List of user ids
        
        ### Returns:
            `dict`: Permissions
        
        ### Example: ::

            permissions = Permissions(...)
            permissions.deny_users([123, 456, ...])
        
        ### Equivalent to: ::

            permissions = Permissions(...)
            permissions.allow_users([123, 456, ...], False)
        """
        self.allow_users(users, False)
        return self.permissions
    
    def allow_roles(self, roles:list, allow:bool=True) -> dict:
        """Allow permissions for a list of role ids

        ### Args:
            `roles (list)`: List of role ids
            `allow (bool, optional)`: Whether to allow or deny permission. Defaults to True.
        
        ### Returns:
            `dict`: Permissions
        
        ### Example: ::

            permissions = Permissions(...)
            permissions.allow_roles([123, 456, ...])
        """
        for id in self.guild_ids:
            for role in roles:
                self.permissions[id].append(create_permission(role, SlashCommandPermissionType.ROLE, allow))
        return self.permissions
    
    def allow_only_roles(self, roles:list) -> dict:
        """Deny permissions for @everyone and allow them to a list of role ids

        ### Args:
            `roles (list)`: List of role ids
        
        ### Returns:
            `dict`: Permissions
        
        ### Example: ::

            permissions = Permissions(...)
            permissions.allow_only_roles([123, 456, ...])
        
        ### Equivalent to: ::

            permissions = Permissions(...)
            permissions.everyone_permission(False)
            permissions.allow_roles([123, 456, ...])
        """
        self.everyone_permission(False)
        self.allow_roles(roles)
        return self.permissions
    
    def deny_roles(self, roles:list[int]) -> dict:
        """Deny permissions to a list of role ids, same as `allow_roles(..., False)`

        ### Args:
            `roles (list[int])`: List of role ids
        
        ### Returns:
            `dict`: Permissions
        
        ### Example: ::

            permissions = Permissions(...)
            permissions.deny_roles([123, 456, ...])
        
        ### Equivalent to: ::

            permissions = Permissions(...)
            permissions.allow_roles([123, 456, ...], False)
        """
        self.allow_roles(roles, False)
        return self.permissions


from typing import Any, Coroutine, Union
import discord
from discord.emoji import Emoji
from discord.message import Message
from discord.partial_emoji import PartialEmoji
from discord_slash.context import ComponentContext
from discord_slash.model import ButtonStyle
from discord_slash.utils.manage_components import create_actionrow, create_button, wait_for_component

def button(style:Union[int, str, ButtonStyle]="PRIMARY", label:Union[str, None]=None, emoji:Union[Emoji, PartialEmoji, str, None]=None, custom_id:Union[str, None]=None, url:Union[str, None]=None, disabled:bool=False) -> dict:
    """Creates a button for use within an action row

    ### Args:
        `style (Union[int, str, ButtonStyle], optional)`: The style of the button. Defaults to "PRIMARY".
        `label (Union[str, None], optional)`: The label of the button. Defaults to None.
        `emoji (Union[Emoji, PartialEmoji, str, None], optional)`: The emoji of the button. Defaults to None.
        `custom_id (Union[str, None], optional)`: The id of the button. Needed for non-link buttons. Defaults to None.
        `url (Union[str, None], optional)`: The URL of the button. Needed for link buttons. Defaults to None.
        `disabled (bool, optional)`: Whether the button is disabled or not. Defaults to False.

    ### Returns:
        dict: Button
    
    ### Example: ::

        my_buttons = buttons(
            button(label="Hi!")
        )
    
    ### Equivalent to: ::

        my_buttons = create_action_row(
            create_button(style=ButtonStyle.primary, label="Hi!")
        )
    """
    if isinstance(style, str):
        style = style.lower()
        if url:
            style = "URL"
        style = ButtonStyle.__dict__[style]
    return create_button(style, label, emoji, custom_id, url, disabled)

def buttons(*button_list:list[dict]) -> dict:
    """Creates an action row for buttons

    ### Args:
        `button_list`: List of buttons to go within the action row

    ### Returns:
        `dict`: Action row
    
    ### Example: ::

        my_buttons = buttons(
            button(label="Hi!")
        )
    
    ### Equivalent to: ::

        my_buttons = create_action_row(
            create_button(style=ButtonStyle.primary, label="Hi!")
        )
    """
    return create_actionrow(*button_list)

async def wait_button(client:discord.Client, buttons:Union[str, dict, list], messages:Union[Message, int, list, None]=None, check=None, timeout=None):
    """Waits for a button interaction. Alternative to `wait_for_component`.

    ### Args:
        `client (discord.Client)`: The client/bot object.
        `buttons (Union[str, dict, list])`: Custom ID to check for, or button dict (buttons or button) or list of previous two.
        `messages (Union[Message, int, list, None], optional)`: The message object to check for, or the message ID or list of the previous two. Defaults to None.
        `check ([type], optional)`: Optional check function. Must take `ComponentContext` as the first parameter. Defaults to None.
        `timeout ([type], optional)`: The number of seconds to wait before timing out and raising `asyncio.TimeoutError`. Defaults to None.

    ### Raises:
        `asyncio.TimeoutError`
    
    ### Example: ::

        button_ctx = await wait_button(bot, my_buttons)
    
    ### Equivalent to: ::

        button_ctx = await wait_for_component(bot, components=my_buttons)
    """
    return await wait_for_component(client, messages, buttons, check, timeout)



from typing import Union
from discord_slash.utils.manage_commands import create_choice, create_option

def option(name:str, description:str, type:Union[int, type]=3, required:bool=True, choices:list=[]):
    """Decorator to add an option to a slash command

    ### Example: ::

        @option("option1", "My option's description")
        @option("option2", "Another description", required=False)
        @slash.slash(...)
    
    ### Equivalent to: ::

        @slash.slash(..., options=[
            create_option("option1", "My option's description", 3, True),
            create_option("option2", "Another description", 3, False)
        ])
    
    ### Args:
        `name (str)`: Option's name
        `description (str)`: Option's description
        `type (Union[int, type], optional)`: Option's type. Defaults to 3.
        `required (bool, optional)`: Should require this option or not. Defaults to True.
        `choices (list, optional)`: Option's choices. Defaults to [].
    """
    def wrapper(cmd):
        for i, x in enumerate(choices):
            if isinstance(x, tuple):
                choices[i] = create_choice(x[0], x[1])
            else:
                choices[i] = create_choice(x, x)
        if not hasattr(cmd, "__options__"):
            cmd.__options__ = True
            cmd.options = []
        cmd.options.insert(0, create_option(name, description, type, required, choices))
        print(cmd.options)
        return cmd
    return wrapper


from codecs import open
from setuptools import find_packages, setup

with open("README.md", "r", encoding="UTF-8") as f:
    README = f.read()

setup(
    name="discord-interactions-styled",
    version="0.4.1",
    author="gammx",
    author_email="gammxplus@gmail.com",
    description="A simple set of tools to write easier code using discord-py-interactions",
    install_requires=["discord.py", "discord-py-slash-command"],
    license="MIT License",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/discord-styled/discord-interactions-styled",
    packages=find_packages(),
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
)


from typing import Union
from discord_slash.model import SlashCommandPermissionType
from discord_slash.utils.manage_commands import create_permission

def permissions(permissions:dict):
    """Apply a slash command permissions template.

    ### Args:
        `permissions (dict)`: Permissions template
    
    ### Example: ::

        @slash.slash(...)
        @permissions(my_permissions)

    """
    def wrapper(cmd):
        cmd.__permissions__ = permissions
        return cmd
    return wrapper

def prepare_command(cmd, guild_id:Union[int, list[int]]):
    """Prepare command adding permissions param and guild_id key if needed

    ### Args:
        `cmd`: Command to prepare
        `guild_id (int, list[int])`: Guild id(s) to prepare

    ### Returns:
        `cmd`: Command with `__permissions__` and `__permissions__[guild_id]`
    """
    if not getattr(cmd, "__permissions__", None):
        cmd.__permissions__ = {}
    if isinstance(guild_id, list):
        for id in guild_id:
            if id not in cmd.__permissions__:
                cmd.__permissions__[id] = []
    else:
        if guild_id not in cmd.__permissions__:
            cmd.__permissions__[guild_id] = []
    return cmd

def _set_everyone_permission(cmd, guild_id:Union[int, list[int]], allow:bool=False):
    """Deny permission for @everyone

    ### Args:
        `cmd`: Command
        `guild_id (int, list[int])`: Guild id(s) to apply permission
        `allow` (bool, optional): Allow or deny

    ### Returns:
        `cmd`: Command with @everyone denied permission
    """
    if isinstance(guild_id, list):
        for id in guild_id:
            cmd.__permissions__[id].append(create_permission(id, SlashCommandPermissionType.ROLE, False))
    else:
        cmd.__permissions__[guild_id].append(create_permission(guild_id, SlashCommandPermissionType.ROLE, allow))
    return cmd

def deny_all(guild_id:Union[int, list[int]]):
    """Decorator, deny permissions for @everyone

    ### Args:
        `guild_id (int, list[int])`: Id(s) of guild to apply permission

    ### Example: ::

        @slash.slash(...)
        @deny_all(123)
    
    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(123, SlashCommandPermissionType.ROLE, False)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        cmd = _set_everyone_permission(cmd, guild_id)
        return cmd
    return wrapper

def allow_all(guild_id:Union[int, list[int]]):
    """Decorator, allow permissions for @everyone

    ### Args:
        `guild_id (int, list[int])`: Id(s) of guild to apply permission

    ### Example: ::

        @slash.slash(...)
        @allow_all(123)
    
    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(123, SlashCommandPermissionType.ROLE, True)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        cmd = _set_everyone_permission(cmd, guild_id, True)
        return cmd
    return wrapper

# ROLE PERMISSIONS ------>

def _allow_role(cmd, guild_id:Union[int, list[int]], role_id:int, allow:bool=True):
    """Create ROLE type permission

    ### Args:
        `cmd`: Command
        `guild_id (int, list[int])`: Guild id to apply permission
        `role_id (int)`: Role id to apply permission
        `allow (bool, optional)`: Allow or deny permission. Defaults to True.

    ### Returns:
        `cmd`: Command with permission created
    """
    if isinstance(guild_id, list):
        for id in guild_id:
            cmd.__permissions__[id].append(
                create_permission(role_id, SlashCommandPermissionType.ROLE, allow)
            )
    else:
        cmd.__permissions__[guild_id].append(
            create_permission(role_id, SlashCommandPermissionType.ROLE, allow)
        )
    return cmd

def only_allow_roles(guild_id:Union[int, list[int]], roles:list[int]):
    """Decorator, deny permissions for @everyone and allow only for selected roles

    ### Args:
        `guild_id (int, list[int])`: Id(s) of guild to apply permissions
        `roles (list[int])`: List of role ids

    ### Example: ::

        @slash.slash(...)
        @only_allow_roles(123, [456, 654])
    
    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(123, SlashCommandPermissionType.ROLE, False),
                create_permission(456, SlashCommandPermissionType.ROLE, True),
                create_permission(654, SlashCommandPermissionType.ROLE, True)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        cmd = _set_everyone_permission(cmd, guild_id)
        for role in roles:
            cmd = _allow_role(cmd, guild_id, role)
        return cmd
    return wrapper

def allow_roles(guild_id:Union[int, list[int]], roles:list[int]):
    """Decorator, allow access for selected roles

    ### Args:
        `guild_id (int)`: Id(s) of guild to apply permissions
        `roles (list[int])`: List of role ids

    ### Example: ::

        @slash.slash(...)
        @just_allow_roles(123, [456, 654])
    
    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(456, SlashCommandPermissionType.ROLE, True),
                create_permission(654, SlashCommandPermissionType.ROLE, True)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        for role in roles:
            cmd = _allow_role(cmd, guild_id, role)
        return cmd
    return wrapper

def deny_roles(guild_id:Union[int, list[int]], roles:list[int]):
    """Decorator, deny permissions for selected roles

    ### Args:
        `guild_id (int, list[int])`: Id(s) of the guild to apply permission
        `roles (list[int])`: List of role ids

    ### Example: ::

        @slash.slash(...)
        @deny_roles(123, [456, 654])

    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(456, SlashCommandPermissionType.ROLE, False),
                create_permission(654, SlashCommandPermissionType.ROLE, False)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        for role in roles:
            cmd = _allow_role(cmd, guild_id, role, False)
        return cmd
    return wrapper

# USER PERMISSIONS ------>

def _allow_user(cmd, guild_id:Union[int, list[int]], user_id:int, allow:bool=True):
    """Create a USER type permission

    ### Args:
        `cmd`: Command
        `guild_id (int, list[int])`: Guild id(s) to apply permission
        `user_id (int)`: User id to apply permission
        `allow (bool, optional)`: Allow or deny permission. Defaults to True.

    Returns:
        `cmd`: Command with permission created
    """
    if isinstance(guild_id, list):
        for id in guild_id:
            cmd.__permissions__[id].append(
                create_permission(user_id, SlashCommandPermissionType.USER, allow)
            )
    else:
        cmd.__permissions__[guild_id].append(
            create_permission(user_id, SlashCommandPermissionType.USER, allow)
        )
    return cmd

def allow_users(guild_id:Union[int, list[int]], users:list[int]):
    """Decorator, Allow access for selected users

    ### Args:
        `guild_id (int, list[int])`: Guild id(s) to apply permission
        `users (list[int])`: List of user ids
    
    ### Example: ::

        @slash.slash(...)
        @allow_users(123, [456, 654])

    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(456, SlashCommandPermissionType.USER, True),
                create_permission(654, SlashCommandPermissionType.USER, True)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        for user in users:
            _allow_user(cmd, guild_id, user)
        return cmd
    return wrapper

def deny_users(guild_id:Union[int, list[int]], users:list[int]):
    """Decorator, Deny access for selected users

    ### Args:
        `guild_id (int)`: Guild id(s) to apply permission
        `users (list[int])`: List of user ids
    
    ### Example: ::

        @slash.slash(...)
        @deny_users(123, [456, 654])

    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(456, SlashCommandPermissionType.USER, False),
                create_permission(654, SlashCommandPermissionType.USER, False)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        for user in users:
            _allow_user(cmd, guild_id, user, False)
        return cmd
    return wrapper

def only_allow_users(guild_id:Union[int, list[int]], users:list[int]):
    """Decorator, Allow access for selected users

    ### Args:
        `guild_id (int, list[int])`: Guild id(s) to apply permission
        `users (list[int])`: List of user ids
    
    ### Example: ::

        @slash.slash(...)
        @only_allow_users(123, [456, 654])

    ### Equivalent to: ::

        @slash.slash(..., permissions={
            123: [
                create_permission(123, SlashCommandPermissionType.USER, False),
                create_permission(456, SlashCommandPermissionType.USER, True),
                create_permission(654, SlashCommandPermissionType.USER, True)
            ]
        })
    """
    def wrapper(cmd):
        cmd = prepare_command(cmd, guild_id)
        cmd = _set_everyone_permission(cmd, guild_id)
        for user in users:
            _allow_user(cmd, guild_id, user)
        return cmd
    return wrapper


"""
█▓▒▒░░░ discord-interactions-styled ░░░▒▒▓█

Simple set of tools to write easier code using discord-py-interactions

copyright: (c) 2021-present gammx
license: MIT
"""

from . import slash
from . import permissions
from . import buttons